@@ -105,11 +105,35 @@ module Concrete = struct
     | Node (l, _, _, _, _) -> min_binding l
     | Empty -> raise Not_found
 
+  let get_root = function
+    | Empty -> raise Not_found
+    | Node (_, k, v, _, _) -> k, v
+
+  let pop_min_binding s =
+    let mini = ref (get_root s) in
+    let rec loop = function
+      | Empty -> assert(false)
+      | Node(Empty, k, v, r, _) -> mini := (k, v); r
+      | Node(l, k, v, r, _) -> bal (loop l) k v r
+    in
+    let others = loop s in
+    (!mini, others)
+
   let rec max_binding = function
     | Node (_, k, v, Empty, _) -> k, v
     | Node (_, _, _, r, _) -> max_binding r
     | Empty -> invalid_arg "PMap.max_binding: empty tree"
 
+  let pop_max_binding s =
+    let maxi = ref (get_root s) in
+    let rec loop = function
+      | Empty -> assert(false)
+      | Node (l, k, v, Empty, _) -> maxi := (k, v); l
+      | Node (l, k, v, r, _) -> bal l k v (loop r)
+    in
+    let others = loop s in
+    (!maxi, others)
+
   let rec remove_min_binding = function
     | Node (Empty, _, _, r, _) -> r
     | Node (l, k, v, r, _) -> bal (remove_min_binding l) k v r
@@ -160,6 +184,23 @@ module Concrete = struct
       | Empty -> Empty in
     loop map
 
+  let update k1 k2 v2 cmp map =
+    if cmp k1 k2 <> 0 then
+      add k2 v2 cmp (remove k1 cmp map)
+    else
+      let rec loop = function
+        | Empty -> raise Not_found
+        | Node(l, k, v, r, h) ->
+          let c = cmp k k1 in
+          if c = 0 then
+            Node(l, k2, v2, r, h)
+          else if c < 0 then
+            Node(loop l, k, v, r, h)
+          else
+            Node(l, k, v, loop r, h)
+      in
+      loop map
+
   let mem x cmp map =
     let rec loop = function
       | Node (l, k, _v, r, _) ->
@@ -210,6 +251,32 @@ module Concrete = struct
         loop (f k v (loop acc l)) r in
     loop acc map
 
+  exception Found
+
+  let at_rank_exn i m =
+    if i < 0 then invalid_arg "Map.at_rank_exn: i < 0";
+    let res = ref (get_root m) in (* raises Not_found if empty *)
+    try
+      let (_: int) =
+        foldi (fun k v j ->
+            if j <> i then j + 1
+            else begin
+              res := (k, v);
+              raise Found
+            end
+          ) m 0
+      in
+      invalid_arg "Map.at_rank_exn: i >= (Map.cardinal s)"
+    with Found -> !res
+
+  (*$T at_rank_exn
+    (empty |> add 1 true |> at_rank_exn 0) = (1, true)
+    (empty |> add 1 true |> add 2 false |> at_rank_exn 1) = (2, false)
+    try ignore(at_rank_exn (-1) empty); false with Invalid_argument _ -> true
+    try ignore(at_rank_exn 0 empty); false with Not_found -> true
+    try ignore(add 1 true empty |> at_rank_exn 1); false with Invalid_argument _ -> true
+  *)
+
   let singleton x d = Node(Empty, x, d, Empty, 1)
 
   (* beware : those two functions assume that the added k is *strictly*
@@ -652,6 +719,7 @@ sig
   val is_empty: 'a t -> bool
   val cardinal: 'a t -> int
   val add: key -> 'a -> 'a t -> 'a t
+  val update: key -> key -> 'a -> 'a t -> 'a t
   val find: key -> 'a t -> 'a
   val remove: key -> 'a t -> 'a t
   val modify: key -> ('a -> 'a) -> 'a t -> 'a t
@@ -672,7 +740,9 @@ sig
   val keys : _ t -> key BatEnum.t
   val values: 'a t -> 'a BatEnum.t
   val min_binding : 'a t -> (key * 'a)
+  val pop_min_binding: 'a t -> (key * 'a) * 'a t
   val max_binding : 'a t -> (key * 'a)
+  val pop_max_binding: 'a t -> (key * 'a) * 'a t
   val choose : 'a t -> (key * 'a)
   val split : key -> 'a t -> ('a t * 'a option * 'a t)
   val partition : (key -> 'a -> bool) -> 'a t -> 'a t * 'a t
@@ -744,6 +814,7 @@ struct
   let backwards t = Concrete.backwards (impl_of_t t)
   let keys t = Concrete.keys (impl_of_t t)
   let values t = Concrete.values (impl_of_t t)
+  let update k1 k2 v2 t = t_of_impl (Concrete.update k1 k2 v2 Ord.compare (impl_of_t t))
 
   let of_enum e = t_of_impl (Concrete.of_enum Ord.compare e)
 
@@ -774,7 +845,13 @@ struct
   let for_all f t = Concrete.for_all f (impl_of_t t)
 
   let min_binding t = Concrete.min_binding (impl_of_t t)
+  let pop_min_binding t =
+    let mini, rest = Concrete.pop_min_binding (impl_of_t t) in
+    (mini, t_of_impl rest)
   let max_binding t = Concrete.max_binding (impl_of_t t)
+  let pop_max_binding t =
+    let maxi, rest = Concrete.pop_max_binding (impl_of_t t) in
+    (maxi, t_of_impl rest)
 
   let choose t = Concrete.choose (impl_of_t t)
 
@@ -847,6 +924,8 @@ let is_empty x = x = Concrete.Empty
 
 let add x d m = Concrete.add x d Pervasives.compare m
 
+let update k1 k2 v2 m = Concrete.update k1 k2 v2 Pervasives.compare m
+
 let find x m = Concrete.find x Pervasives.compare m
 
 (*$T add; find
@@ -858,6 +937,20 @@ let find x m = Concrete.find x Pervasives.compare m
   empty |> add 2 'y' |> add 1 'x' |> find 2 = 'y'
 *)
 
+(*$T pop_min_binding
+  (empty |> add 1 true |> pop_min_binding) = ((1, true), empty)
+  (empty |> add 1 true |> add 2 false |> pop_min_binding) = \
+  ((1, true), add 2 false empty)
+  try ignore (pop_min_binding empty); false with Not_found -> true
+*)
+
+(*$T pop_max_binding
+  (empty |> add 1 true |> pop_max_binding) = ((1, true), empty)
+  (empty |> add 1 true |> add 2 false |> pop_max_binding) = \
+  ((2, false), add 1 true empty)
+  try ignore (pop_max_binding empty); false with Not_found -> true
+*)
+
 (*$Q find ; add
   (Q.list Q.small_int) (fun xs -> \
   let of_list xs y m0 = List.fold_left (fun acc x -> add x y acc) m0 xs in \
@@ -880,6 +973,7 @@ let map = Concrete.map
 let mapi = Concrete.mapi
 let fold = Concrete.fold
 let foldi = Concrete.foldi
+let at_rank_exn = Concrete.at_rank_exn
 
 (*$Q foldi
   (Q.list Q.small_int) (fun xs -> \
@@ -912,6 +1006,8 @@ let filter_map f t = Concrete.filter_map f t Pervasives.compare
 let choose = Concrete.choose
 let max_binding = Concrete.max_binding
 let min_binding = Concrete.min_binding
+let pop_min_binding = Concrete.pop_min_binding
+let pop_max_binding = Concrete.pop_max_binding
 
 let singleton k v = Concrete.singleton k v
 
@@ -1011,6 +1107,9 @@ module PMap = struct (*$< PMap *)
   let add x d m =
     { m with map = Concrete.add x d m.cmp m.map }
 
+  let update k1 k2 v2 m =
+    { m with map = Concrete.update k1 k2 v2 m.cmp m.map }
+
   let find x m =
     Concrete.find x m.cmp m.map
 
@@ -1023,6 +1122,12 @@ module PMap = struct (*$< PMap *)
     empty |> add 2 'y' |> add 1 'x' |> find 2 = 'y'
   *)
 
+  (*$T update
+    add 1 false empty |> update 1 1 true |> find 1
+    add 1 false empty |> update 1 2 true |> find 2
+    try ignore (update 1 1 false empty); false with Not_found -> true
+  *)
+
   (*$Q find ; add
     (Q.list Q.small_int) (fun xs -> \
     let of_list xs y m0 = List.fold_left (fun acc x -> add x y acc) m0 xs in \
@@ -1063,6 +1168,9 @@ module PMap = struct (*$< PMap *)
     foldi (fun x _y acc -> x :: acc) m [] |> List.rev = List.sort_unique Int.compare xs)
   *)
 
+  let at_rank_exn i m =
+    Concrete.at_rank_exn i m.map
+
   let enum t = Concrete.enum t.map
 
   (*$Q keys
@@ -1091,6 +1199,12 @@ module PMap = struct (*$< PMap *)
 
   let max_binding t = Concrete.max_binding t.map
   let min_binding t = Concrete.min_binding t.map
+  let pop_min_binding m = 
+    let mini, rest = Concrete.pop_min_binding m.map in
+    (mini, { m with map = rest })
+  let pop_max_binding m = 
+    let maxi, rest = Concrete.pop_max_binding m.map in
+    (maxi, { m with map = rest })
 
   let singleton ?(cmp = Pervasives.compare) k v =
     { cmp = cmp; map = Concrete.singleton k v }
